//  @flow
import 'require-ensure';
import React from 'react';
import { StaticRouter } from 'react-router';
import { renderToString } from 'react-dom/server';

import type { $Request, $Response } from 'express';
import type { IConfig } from '@/IConfig';
import type { IApp } from '@/IApp';

import {
  matchRoute,
  fetchInitialProps,
  getParamsFromUrl,
  getRootString,
  getMetaData,
} from './../utils';

export default (app: IApp, config: IConfig) => async (
  req: $Request,
  res: $Response,
  next: Function,
) => {
  const _app = app;
  let tmpl = app.locals.html;

  //  The props we'll inject into the root component
  const serverProps = {};

  //  Grab the things we need from the userConfig
  const { root: Root, routes } = config;

  //  Resolve the current route from the supplied
  //  array of SPA routes.
  const { meta, path, component } = matchRoute(routes, req.url);

  //  TODO handle 404's and broken promises
  if (!path) {
    res.status(404);
  }

  //  Fetch any initialProps to populate route data
  const urlParams = getParamsFromUrl(path, req.url);
  serverProps[req.url] = await fetchInitialProps(component.getInitialProps, urlParams);

  //  Get the contextRoute meta data and
  //  populate the html server-side
  const metaData = getMetaData(meta, urlParams);
  Object.keys(metaData).forEach((key) => {
    tmpl = tmpl.replace(`{meta.${key}}`, metaData[key]);
  });

  //  Replaces the <div id="root" /> element
  //  with the server rendered application
  _app.locals.output = tmpl.replace(
    '<div id="root" />',
    `<div id="root">
      ${getRootString(renderToString(
        <StaticRouter location={req.url} context={{}}>
          <Root {...serverProps} />
        </StaticRouter>,
      ), serverProps)}
    </div>`,
  );

  next();
};
