import { expect } from 'chai';

import getMetaData from './getMetaData';

describe('getMetaData()', () => {
  it('should return the meta object from the route context', () => {
    const meta = {
      title: 'some title',
      description: 'some description',
    };

    expect(getMetaData(meta, null)).to.eql({
      title: 'some title',
      description: 'some description',
    });
  });

  it('should replace a token in the meta object and return it', () => {
    const meta = {
      title: 'some title for {pageId}',
      description: 'some description for {pageId}',
    };

    const urlParams = { pageId: 'blah' };

    expect(getMetaData(meta, urlParams)).to.eql({
      title: 'some title for blah',
      description: 'some description for blah',
    });
  });

  it('should replace all tokens in the meta object and return it', () => {
    const meta = {
      title: 'some title for {pageId} {anotherId}',
      description: 'some description for {pageId} {anotherId}',
    };

    const urlParams = {
      pageId: 'blah',
      anotherId: 'foo',
    };

    expect(getMetaData(meta, urlParams)).to.eql({
      title: 'some title for blah foo',
      description: 'some description for blah foo',
    });
  });
});
