//  @flow
import ExtractTextPlugin from 'extract-text-webpack-plugin';
import defaultConfig from './../defaultConfig';
import webpack from 'webpack';
import postcssImport from 'postcss-import';
import postcssCssNext from 'postcss-cssnext';

import type { IConfig } from '../../flow/IConfig';

export default ({
  localIdentName = defaultConfig.localIdentName,
}: IConfig) => (
  [
    {
      test: /\.js?$/,
      exclude: /node_modules/,
      loader: 'babel-loader',
      options: {
        babelrc: false,
        presets: ['es2015', 'react', 'stage-0'],
        plugins: ['transform-regenerator'],
      },
    },
    {
      test: /\.(css|scss)?$/,
      exclude: /node_modules/,
      loader: ExtractTextPlugin.extract([
        {
          loader: 'css-loader',
          options: {
            context: './',
            modules: true,
            importLoaders: 1,
            localIdentName,
          },
        },
        {
          loader: 'postcss-loader',
          options: {
            plugins: [
              postcssImport({
                addDependencyTo: webpack,
              }),
              postcssCssNext(),
            ],
          },
        },
      ]),
    },
    {
      test: /\.(png|svg)?$/,
      exclude: /node_modules/,
      use: {
        loader: 'file-loader',
        options: {
          name: '[name].[hash].[ext]',
          outputPath: '/images',
        },
      },
    },
  ]
);
