//  @flow
import path from 'path';

import getLoaders from './loaders.config';
import getPlugins from './plugins.config';
import defaultConfig from './../defaultConfig';

import type { IConfig } from '../../flow/IConfig';

export default (config: IConfig) => {
  const {
    context = path.resolve(process.cwd(), './src'),
    entry,
    distOutput = defaultConfig.distOutput,
    publicPath = defaultConfig.publicPath,
    jsPath = defaultConfig.jsPath,
    isDevEnvironment,
  } = config;

  if (!entry) {
    throw new Error('No entry files are specified');
  }

  return {
    context,

    entry: {
      ...entry,
      ...(isDevEnvironment) && {
        hotReload: 'webpack-hot-middleware/client?reload=true',
      },
    },

    output: {
      filename: (isDevEnvironment)
        ? `${jsPath}/[name].bundle.js`
        : `${jsPath}/[name].[chunkhash:8].bundle.js`,
      chunkFilename: (isDevEnvironment)
        ? `${jsPath}/[name].chunk.js`
        : `${jsPath}/[name].[chunkhash:8].chunk.js`,
      path: path.join(process.cwd(), distOutput, publicPath),
      publicPath,
    },

    //  Add devServer and source maps
    //  when running in development
    ...(isDevEnvironment) && {
      devServer: {
        contentBase: publicPath,
        hot: true,
        inline: true,
        historyApiFallback: true,
      } && {
        devtool: 'cheap-module-source-map',
      },
    },

    module: {
      rules: getLoaders(config),
    },
    plugins: getPlugins(config),
  };
};
